@@ -25,7 +25,7 @@ SDC_KEYS = {'id': 'integer', 'name': 'string', 'myshopify_domain': 'string'}
 def initialize_shopify_client():
     api_key = Context.config['api_key']
     shop = Context.config['shop']
-    version = '2022-07'
+    version = '2024-01'
     session = shopify.Session(shop, version, api_key)
     shopify.ShopifyResource.activate_session(session)
 
@@ -47,7 +47,7 @@ def load_schemas():
     for filename in os.listdir(get_abs_path('schemas')):
         path = get_abs_path('schemas') + '/' + filename
         schema_name = filename.replace('.json', '')
-        with open(path) as file:
+        with open(path, encoding='UTF-8') as file:
             schemas[schema_name] = json.load(file)
 
     return schemas
@@ -74,7 +74,7 @@ def load_schema_references():
     shared_schema_path = get_abs_path('schemas/')
 
     refs = {}
-    with open(os.path.join(shared_schema_path, shared_schema_file)) as data_file:
+    with open(os.path.join(shared_schema_path, shared_schema_file), encoding='UTF-8') as data_file:
         refs[shared_schema_file] = json.load(data_file)
 
     return refs
@@ -137,6 +137,12 @@ def sync():
     shop_attributes = initialize_shopify_client()
     sdc_fields = {"_sdc_shop_" + x: shop_attributes[x] for x in SDC_KEYS}
 
+    # If there is a currently syncing stream bookmark, shuffle the
+    # stream order so it gets sync'd first
+    currently_sync_stream_name = Context.state.get('bookmarks', {}).get('currently_sync_stream')
+    if currently_sync_stream_name:
+        shuffle_streams(currently_sync_stream_name)
+
     # Emit all schemas first so we have them for child streams
     for stream in Context.catalog["streams"]:
         if Context.is_selected(stream["tap_stream_id"]):
@@ -146,12 +152,6 @@ def sync():
                                 bookmark_properties=stream["replication_key"])
             Context.counts[stream["tap_stream_id"]] = 0
 
-    # If there is a currently syncing stream bookmark, shuffle the
-    # stream order so it gets sync'd first
-    currently_sync_stream_name = Context.state.get('bookmarks', {}).get('currently_sync_stream')
-    if currently_sync_stream_name:
-        shuffle_streams(currently_sync_stream_name)
-
     # Loop over streams in catalog
     for catalog_entry in Context.catalog['streams']:
         stream_id = catalog_entry['tap_stream_id']
