@@ -1,14 +1,15 @@
 import datetime
 import functools
-import math
-import sys
 import socket
+from urllib.error import URLError
+import http
 import backoff
 import pyactiveresource
 import pyactiveresource.formats
 import simplejson
 import singer
 from singer import metrics, utils
+from singer.utils import strptime_to_utc
 from tap_shopify.context import Context
 
 LOGGER = singer.get_logger()
@@ -94,18 +95,6 @@ def retry_handler(details):
     LOGGER.info("Received 500 or retryable error -- Retry %s/%s",
                 details['tries'], MAX_RETRIES)
 
-#pylint: disable=unused-argument
-def retry_after_wait_gen(**kwargs):
-    # This is called in an except block so we can retrieve the exception
-    # and check it.
-    exc_info = sys.exc_info()
-    resp = exc_info[1].response
-    # Retry-After is an undocumented header. But honoring
-    # it was proven to work in our spikes.
-    # It's been observed to come through as lowercase, so fallback if not present
-    sleep_time_str = resp.headers.get('Retry-After', resp.headers.get('retry-after'))
-    yield math.floor(float(sleep_time_str))
-
 # boolean function to check if the error is 'timeout' error or not
 def is_timeout_error(error_raised):
     """
@@ -118,6 +107,10 @@ def is_timeout_error(error_raised):
     return True
 
 def shopify_error_handling(fnc):
+    @backoff.on_exception(backoff.expo,
+                          (http.client.IncompleteRead, ConnectionResetError),
+                          max_tries=MAX_RETRIES,
+                          factor=2)
     @backoff.on_exception(backoff.expo, # timeout error raise by Shopify
                           (pyactiveresource.connection.Error, socket.timeout),
                           giveup=is_timeout_error,
@@ -126,11 +119,17 @@ def shopify_error_handling(fnc):
     @backoff.on_exception(backoff.expo,
                           (pyactiveresource.connection.ServerError,
                            pyactiveresource.formats.Error,
-                           simplejson.scanner.JSONDecodeError),
+                           simplejson.scanner.JSONDecodeError,
+                           URLError),
                           giveup=is_not_status_code_fn(range(500, 599)),
                           on_backoff=retry_handler,
                           max_tries=MAX_RETRIES)
-    @backoff.on_exception(retry_after_wait_gen,
+    @backoff.on_exception(backoff.expo,
+                          pyactiveresource.connection.ResourceNotFound,
+                          giveup=is_not_status_code_fn([404]),
+                          on_backoff=retry_handler,
+                          max_tries=MAX_RETRIES)
+    @backoff.on_exception(backoff.expo,
                           pyactiveresource.connection.ClientError,
                           giveup=is_not_status_code_fn([429]),
                           on_backoff=leaky_bucket_handler,
@@ -180,6 +179,10 @@ class Stream():
                                    self.name,
                                    'since_id')
 
+    def get_updated_at_max(self):
+        updated_at_max = Context.state.get('bookmarks', {}).get(self.name, {}).get('updated_at_max')
+        return utils.strptime_with_tz(updated_at_max) if updated_at_max else None
+
     def update_bookmark(self, bookmark_value, bookmark_key=None):
         # NOTE: Bookmarking can never be updated to not get the most
         # recent thing it saw the next time you run, because the querying
@@ -213,7 +216,9 @@ class Stream():
         }
 
     def get_objects(self):
+        last_sync_interrupted_at = self.get_updated_at_max()
         updated_at_min = self.get_bookmark()
+        max_bookmark = updated_at_min
 
         stop_time = singer.utils.now().replace(microsecond=0)
         date_window_size = float(Context.config.get("date_window_size", DATE_WINDOW_SIZE))
@@ -231,9 +236,15 @@ class Stream():
             # think it has something to do with how the API treats
             # microseconds on its date windows. Maybe it's possible to
             # drop data due to rounding errors or something like that?
-            updated_at_max = updated_at_min + datetime.timedelta(days=date_window_size)
-            if updated_at_max > stop_time:
-                updated_at_max = stop_time
+            # If last sync was interrupted, set updated_at_max to
+            # updated_at_max bookmarked in the interrupted sync.
+            # This will make sure that records with lower id than since_id
+            # which got updated later won't be missed
+            updated_at_max = (last_sync_interrupted_at
+                              or updated_at_min + datetime.timedelta(days=date_window_size))
+            last_sync_interrupted_at = None
+
+            updated_at_max = min(updated_at_max, stop_time)
             while True:
                 status_key = self.status_key or "status"
                 query_params = self.get_query_params(since_id,
@@ -251,6 +262,9 @@ class Stream():
                         # since_id parameter.
                         raise OutOfOrderIdsError("obj.id < since_id: {} < {}".format(
                             obj.id, since_id))
+                    replication_value = strptime_to_utc(getattr(obj, self.replication_key))
+                    if replication_value > max_bookmark:
+                        max_bookmark = replication_value
                     yield obj
 
                 # You know you're at the end when the current page has
@@ -259,7 +273,9 @@ class Stream():
                     # Save the updated_at_max as our bookmark as we've synced all rows up in our
                     # window and can move forward. Also remove the since_id because we want to
                     # restart at 1.
-                    Context.state.get('bookmarks', {}).get(self.name, {}).pop('since_id', None)
+                    stream_bookmarks = Context.state.get('bookmarks', {}).get(self.name, {})
+                    stream_bookmarks.pop('since_id', None)
+                    stream_bookmarks.pop('updated_at_max', None)
                     self.update_bookmark(utils.strftime(updated_at_max))
                     break
 
@@ -271,10 +287,15 @@ class Stream():
                         objects[-1].id, max([o.id for o in objects])))
                 since_id = objects[-1].id
 
-                # Put since_id into the state.
+                # Put since_id and updated_at_max into the state.
                 self.update_bookmark(since_id, bookmark_key='since_id')
+                self.update_bookmark(utils.strftime(updated_at_max), bookmark_key='updated_at_max')
 
             updated_at_min = updated_at_max
+        bookmark = max(min(stop_time,
+                           max_bookmark),
+                       (stop_time - datetime.timedelta(days=date_window_size)))
+        self.update_bookmark(utils.strftime(bookmark))
 
     def sync(self):
         """Yield's processed SDK object dicts to the caller.
